;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#xbf5dac5cae05ca75a7c9685b5e19d5fa . "./code.cg2js/cg-acache.fasl")
(#x61b9da3ba1586ed9684a41a60792e3de . "./code.cg2js/cg-base.fasl")
(#x2605fb5e2c530d5dc70b60943366adf2 . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#x715b36f1223903b452c8eb8b37333f25 . "./code.cg2js/cg-bitmap-pane.fasl")
(#xe07bc03ecbe95f81d03dd0d3601ca34a . "./code.cg2js/cg-bitmap-stream.fasl")
(#x936e9a083adca8e448ecef99e5c8b0a9 . "./code.cg2js/cg-button.fasl")
(#x3a5355125de25cb1b00c41e7b4838c43 . "./code.cg2js/cg-calendar.fasl")
(#xbe60d70a76e7ec517bbe379a8673dd3d . "./code.cg2js/cg-caret.fasl")
(#x9903c3aefa4a5a442f82461e3461bc49 . "./code.cg2js/cg-chart-or-plot.fasl")
(#xa46429a4b131d6295f6a76f8e030e6bc . "./code.cg2js/cg-chart-widget.fasl")
(#xcfd9d0a538ac2cd1c38f58bd43e23bda . "./code.cg2js/cg-check-box.fasl")
(#x3a893e9e691f1576019043ad6e4240c0 . "./code.cg2js/cg-choice-list.fasl")
(#x773e3ba1b0ec417e0d91892c020dd166 . "./code.cg2js/cg-choose-printer.fasl")
(#xf4a238b3c28294368d71c3c17b52c89b . "./code.cg2js/cg-class-grid.fasl")
(#x6af3ed9ee1ea7c88fd37c605cd93a9b5 . "./code.cg2js/cg-class-slot-grid.fasl")
(#x8fd06491d347f4e78ccc5bf22ceabfea . "./code.cg2js/cg-class-support.fasl")
(#x74653254ef15874b16fa8be6489971ef . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#x559055a4cd0dff87a3da8ca3c6d8a957 . "./code.cg2js/cg-clipboard-stack.fasl")
(#xf23ebe6c4ffd5f94e8886ecaaa386b97 . "./code.cg2js/cg-clipboard.fasl")
(#x0db9eca23c392014db642748c6c56a22 . "./code.cg2js/cg-color-dialog.fasl")
(#x05175e1f761753308a94230bd81695b2 . "./code.cg2js/cg-combo-box.fasl")
(#xf880bf3ba71411e22c48681b4d4132a7 . "./code.cg2js/cg-common-control.fasl")
(#x741aa9a3481045da14b97091b7562c23 . "./code.cg2js/cg-comtab.fasl")
(#x5ffc5a24e54e039e6a17d7b0f3e35699 . "./code.cg2js/cg-cursor-pixmap.fasl")
(#x1f826f408ac92e27fe3ffc2fccd9491c . "./code.cg2js/cg-curve.fasl")
(#x1d59b8c8f0f152e778298dcdd76eb9b2 . "./code.cg2js/cg-dde-utils.fasl")
(#xae0e3508c2f6e511957706fb9dd5a09d . "./code.cg2js/cg-dev.fasl")
(#x9cb862ce1b3792804992d7bd18f2c3cd . "./code.cg2js/cg-dialog-item.fasl")
(#x9539941a323d4343babf28bd88a4d7a4 . "./code.cg2js/cg-directory-dialog-os.fasl")
(#x55ac9cb9f6f3837da6ce82e775df12da . "./code.cg2js/cg-directory-dialog.fasl")
(#xbb976ae581fd7f7884a6d0d2d3b1d9f7 . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#x0c23391667b603aaeff3594598ebad9d . "./code.cg2js/cg-drag-and-drop.fasl")
(#x99656694cf97e379e4ef3a92fc921b65 . "./code.cg2js/cg-drawable-clipboard.fasl")
(#x99f29a807bb0c7b1499c0b57ab75c059 . "./code.cg2js/cg-drawable.fasl")
(#xa39b86e2081bea5973d36f479a3d0d11 . "./code.cg2js/cg-dropping-outline.fasl")
(#xc4523e08302d7697f2d9502a689abf98 . "./code.cg2js/cg-edit-in-place.fasl")
(#xf41e7e7e67792d194ef2f2ab6cf73054 . "./code.cg2js/cg-editable-text.fasl")
(#x5a1747eba18efda9ac6ae5b8d729badd . "./code.cg2js/cg-file-dialog.fasl")
(#xb892d6705d9ea5bf0521f3aa68a71e35 . "./code.cg2js/cg-fill-texture.fasl")
(#x9e15055e3a54f2da1451ad7eba7ac13d . "./code.cg2js/cg-find-string-dialog.fasl")
(#x0e3e2fce2eb683da8a1f6c3ef9c25e3c . "./code.cg2js/cg-font-dialog.fasl")
(#x67b01c52c4f7b276b35dd837bb0a0158 . "./code.cg2js/cg-gesture-emulation.fasl")
(#x7fb6300ecf12cf1925581eafbede4c5c . "./code.cg2js/cg-get-pixmap.fasl")
(#x441791ebbc926f096a61ff23c5f361f2 . "./code.cg2js/cg-get-position.fasl")
(#x24888dcf9ce420b165010fcbe36c67cd . "./code.cg2js/cg-graphics-context.fasl")
(#x15d50d2d89aed46551b50112c37b68c2 . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#xba4f16638cde6e4c586bbabed1bef79e . "./code.cg2js/cg-grid-widget.fasl")
(#x85531d049891c775b5c05a2ee3cca570 . "./code.cg2js/cg-group-box.fasl")
(#x81b8f0582903cf06b04bc782c4a93a68 . "./code.cg2js/cg-header-control.fasl")
(#xb024760db42d98f0b5514ef4aa80cd4f . "./code.cg2js/cg-hotspot.fasl")
(#x65072c4eb5de2720f486df76e793cfff . "./code.cg2js/cg-html-dialog.fasl")
(#x416ddd02e9a5a82c314df20933e5279b . "./code.cg2js/cg-html-widget.fasl")
(#xc731c8464936af02f35761c1897fc737 . "./code.cg2js/cg-icon-pixmap.fasl")
(#xefcf61a861a8d332b5dca9c4c2f2aca1 . "./code.cg2js/cg-icon.fasl")
(#xca818ad624307e2f8795c614e21808b7 . "./code.cg2js/cg-ie.fasl")
(#x6a8093c9c6edfbcae0e897d187a2f28c . "./code.cg2js/cg-intersect-posbox.fasl")
(#x7ac9710a0a7beed212b5a4b573cf5602 . "./code.cg2js/cg-intersect.fasl")
(#xafd77958b1765b6b6d4a5cae442ff1fb . "./code.cg2js/cg-item-list.fasl")
(#x01abd83d786829fec470bbb8c13f8c9a . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#x4c09a883c6677e993534b5b0d12c0349 . "./code.cg2js/cg-lamp.fasl")
(#x477ac0b72bc913fb490ad1b680ec0d6c . "./code.cg2js/cg-layout.fasl")
(#xa5dbe4005ada7973fe6aabe0a9bb0343 . "./code.cg2js/cg-lettered-menu.fasl")
(#x5d2883bec1d6471bcc5937b1c2dd5360 . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#xdc18d3953f6b1f17d51f03c71aa8f0e3 . "./code.cg2js/cg-lisp-text.fasl")
(#xd8cea3df7c262b335104440b380d6ffa . "./code.cg2js/cg-lisp-widget.fasl")
(#x46b8452c75e79d3fdd4a1d89a4cae981 . "./code.cg2js/cg-list-view.fasl")
(#x5dfc0216fdc8f001a8c67463b2f9e399 . "./code.cg2js/cg-mci.fasl")
(#x265c1a252b708fa55e7f2fd966a5e787 . "./code.cg2js/cg-menu-tooltip.fasl")
(#x42e204d40c542d6fdc82e87e75ead135 . "./code.cg2js/cg-menu.fasl")
(#x84b2ed1e50bcb1b0ee7a7a5353f912e0 . "./code.cg2js/cg-message-dialog.fasl")
(#x51e67a3f1e3787b1d81940289237edf1 . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#x675f5cb66bc4c7cba379b60470d233be . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#x649a0a584171439f50d39adacac3feb1 . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#x9379bbcfc15ede2373cd436a4828b43c . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#xf44c67277beb55bad7fb27cd3f772f4b . "./code.cg2js/cg-multi-picture-button.fasl")
(#x5230546c0975ea6bf6b1728cedfa9b9c . "./code.cg2js/cg-nodes.fasl")
(#x1bd3044fe4ae426f7071590c8067ffe8 . "./code.cg2js/cg-object-editor-layout.fasl")
(#xcab716e7d897afba4ac24ff8b5641389 . "./code.cg2js/cg-object-editor.fasl")
(#xa1817dcbfe97876ccacb9cd121521501 . "./code.cg2js/cg-ocx.fasl")
(#xb0b0a79580332c0faa9af1cc72bcda2c . "./code.cg2js/cg-os-widget.fasl")
(#x1dc101ac416d431f9f9f62096a083591 . "./code.cg2js/cg-os-window.fasl")
(#x0d300e2c995d3624b103f1e92549cb71 . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#x749af4c9199451ee9c46555222a2c769 . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#x84610bc31583abfaf5cb5278e0fc7396 . "./code.cg2js/cg-outline.fasl")
(#xed3a0819944d7d89a74c1e9c2f979a6f . "./code.cg2js/cg-palette.fasl")
(#xc8a755a5093d30408770860a0d2b93ab . "./code.cg2js/cg-paren-matching.fasl")
(#x97c51d3787aa53c34e73620b7fddaade . "./code.cg2js/cg-picture-widget-palette.fasl")
(#xc949f6e08e3a41480aca9d7fc95ff807 . "./code.cg2js/cg-picture-widget.fasl")
(#x7181af507327aca9c38424678e0d3fd7 . "./code.cg2js/cg-pixmap-file-io.fasl")
(#x0360e2499c4eeeb759fca85c888cc4ae . "./code.cg2js/cg-pixmap-printing.fasl")
(#x2d4652cdf1326df0e1caa9a6f670f574 . "./code.cg2js/cg-pixmap-rotate.fasl")
(#xb6c56fcef620e4f8beccfd599aaf4841 . "./code.cg2js/cg-pixmap-widget.fasl")
(#x9c2e6a0fa7ba74fc076f81140b865e8d . "./code.cg2js/cg-pixmap.fasl")
(#x45d62152aad8f4a6e7a14cf97e1bffb1 . "./code.cg2js/cg-plot-widget.fasl")
(#x72607269ffb8c5510864bf4718edde50 . "./code.cg2js/cg-printing.fasl")
(#x9acfadca25f48d6169e8a713b6edf473 . "./code.cg2js/cg-progress-indicator.fasl")
(#x1f6807c0e167a751f4dfb5b0a6b0177c . "./code.cg2js/cg-project-window.fasl")
(#x866de4ddfa4e15b18bf31994ea11b653 . "./code.cg2js/cg-property.fasl")
(#x748a34369795ce709f3b4c757bc35f81 . "./code.cg2js/cg-radio-button.fasl")
(#xea0e52595caded42d038347094c5ac5f . "./code.cg2js/cg-rich-edit-pane-clipboard.fasl")
(#xcd3d63ead31f1f9e15c12b0664cc43c4 . "./code.cg2js/cg-rich-edit-pane-printing.fasl")
(#x04d93c20afbe050489afd44071e76988 . "./code.cg2js/cg-rich-edit-pane.fasl")
(#x709cecc7cf309936a6c75fd15572675b . "./code.cg2js/cg-rich-edit.fasl")
(#x28fdca58d3562bbfefe7e7588a1ffcbf . "./code.cg2js/cg-sample-file-menu.fasl")
(#xe1edf95187c4d055ac3620a5ab893b51 . "./code.cg2js/cg-scaling-stream.fasl")
(#x3c35621beaf2fe843d23b0dfb9e2474d . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#xf83fef577ca5c6efadb742563a7e2252 . "./code.cg2js/cg-scroll-bar.fasl")
(#x6a7b2aad4cd2ca07ec4881e4a5d18cc0 . "./code.cg2js/cg-scrolling-static-text.fasl")
(#x3aaacb794bc9459779264fbf5efd5528 . "./code.cg2js/cg-selected-object.fasl")
(#x730cf59108bd51340ffd6ad71a33c545 . "./code.cg2js/cg-shortcut-menu.fasl")
(#xcf7c5ff5f6784102c8a6abdeb8c1eda0 . "./code.cg2js/cg-split-bar.fasl")
(#xc3449aa8f17bed7bbedb27ac90387af8 . "./code.cg2js/cg-static-text.fasl")
(#xe0dc1436d5f3e3db95fef85ce3a5095d . "./code.cg2js/cg-status-bar.fasl")
(#x06e72bdb92dd1bbd7e4298f6ddb0f2a7 . "./code.cg2js/cg-string-dialog.fasl")
(#xa010aa73ef02f8112149ffa2b388a47f . "./code.cg2js/cg-tab-control.fasl")
(#xbb2d39be90ef33ee97d88a1156389653 . "./code.cg2js/cg-template-string.fasl")
(#xdf77e0bc0bbb93dbabaa623e6f9bcf91 . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#x64dcbfa7d4731dda8be732af4168beca . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#xbda0e939b5bc56116aab74c42c8ed8a7 . "./code.cg2js/cg-text-edit-pane.fasl")
(#x504bc8fe3064a84386ae890edf05f760 . "./code.cg2js/cg-text-or-combo.fasl")
(#x989f61b175e361c3b4eea496bd01966e . "./code.cg2js/cg-text-widget.fasl")
(#xc20fc19f253ac41c57b66995c02b00f7 . "./code.cg2js/cg-timer.fasl")
(#x92b034d32ab0a57897ca1844923b21bf . "./code.cg2js/cg-toggling-widget.fasl")
(#x08010eeb304ac2caa9e4337e66c51fbc . "./code.cg2js/cg-toolbar.fasl")
(#x25757742c99818cb12b056f3a8ee71e7 . "./code.cg2js/cg-tooltip.fasl")
(#xbef44a6486186d1a2568efcdb1bf67a1 . "./code.cg2js/cg-touch.fasl")
(#x7d1c7aa90c8c5a517d90fae99b13528f . "./code.cg2js/cg-trackbar.fasl")
(#x15bc0848018f0b15e0a804ed07e6bbe1 . "./code.cg2js/cg-tray.fasl")
(#x42b4f748492253466d4675cb90382a95 . "./code.cg2js/cg-up-down-control.fasl")
(#x99f0ea70ceecd57c72502eadf6fc91c7 . "./code.cg2js/cg-utility-dialog.fasl")
(#x170e6b205b7da97a2db32d4f4a50bef0 . "./code.cg2js/cg-web-browser-dde.fasl")
(#x7eec76b1af4736f3ccd35a04acc82587 . "./code.cg2js/cg-web-browser.fasl")
(#x439a73cc2bd6577de25b59de10d5ca13 . "./code.cg2js/cg-wrap-string.fasl")
(#x459e56b338625316342f83fc10c1d196 . "./code.cg2js/cg-yes-no-list.fasl")
(#xff59787a23b240b998c5998b7d997314 . "./code.cg2js/cg-yes-no-string.fasl")
(#x298a29d97b3ff40fa6e34dfbf8cb05ec . "./code.cg2js/cg.fasl")
(#x69ca0be6d4f57bd8c447444f9e9174c9 . "./code.idejs/ide-acache.fasl")
(#x04ea21d88b5802a8276f537d8240cc87 . "./code.idejs/ide-apropos.fasl")
(#x800bd950074016a8d75994dbf8c384e6 . "./code.idejs/ide-base.fasl")
(#x2daf47be1e47d86fea4d94557604ecc8 . "./code.idejs/ide-class-browser.fasl")
(#x10f45703bf0a157f17688c7440b20631 . "./code.idejs/ide-class-grapher.fasl")
(#xb4126b3d70b50cc6545f016c990ca17b . "./code.idejs/ide-clipboard.fasl")
(#xf6eb6404ded1ffd77452721b451365c9 . "./code.idejs/ide-dev.fasl")
(#x102333178257a1b8b329b70de075a653 . "./code.idejs/ide-editor.fasl")
(#x3f256b62e5fccf04a633029b6ae450f6 . "./code.idejs/ide-find-definitions.fasl")
(#x60858c2d52278d0494e5514a278ac303 . "./code.idejs/ide-find-in-files.fasl")
(#x30d3db1850e725771ca91d6bb4b09e81 . "./code.idejs/ide-form.fasl")
(#x726f575bd34e148eb33128a7cdf5db92 . "./code.idejs/ide-grapher.fasl")
(#x7348bad847ca63c33b9ea373776f0e2d . "./code.idejs/ide-inspector.fasl")
(#x0c099c10ac6fe9a54b8652b8cf2dc61a . "./code.idejs/ide-intro.fasl")
(#x4113d3ec0fbdfb3954a5d7b46ef98950 . "./code.idejs/ide-keyshort.fasl")
(#x4d8a8affb01bf9a63378d1cd32f6a360 . "./code.idejs/ide-navigator.fasl")
(#x2de597828582ac7a3a3d024b53432de2 . "./code.idejs/ide-package-list.fasl")
(#x27d53001ae8c9dbaf7852744264b0200 . "./code.idejs/ide-patch-dialog.fasl")
(#x54d5841185456bf91abf11a0bcab1ce4 . "./code.idejs/ide-process-browser.fasl")
(#xf8ae3f856fa82813216e40bd1dfdac5f . "./code.idejs/ide-project.fasl")
(#xd73d11b6afee7090e732bf31eb971850 . "./code.idejs/ide-trace.fasl")
(#xd600f8651e3d07671fd9721d9cd4e037 . "./code.idejs/ide-window-list.fasl")
(#xcf3521e29528e19f03f1f877a1fecc57 . "./code.idejs/ide.fasl")
(#xdf73f3983b983ba8030e7ee4702b498a . "./cgjs.html")
(#xa0fd27890288fb85b9ee621ed47b2251 . "./ssl-not-found.html")
(#x4e1b21622d7d9cc4e6fc09c7875f8430 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js/" "code.idejs/"))
    (format s "
(let ((path (make-pathname
	     :type \"fasl\"
	     :directory (pathname-directory
			 (merge-pathnames
			  ~s
			  (translate-logical-pathname \"sys:\"))))))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows (excl::internal-message-box msg "CG/JS Installation" :type :ok))
